#include <SoftwareSerial.h>

// Arduino Motor Shield制御用の端子の定義

#define PORT_DIR_A 12
#define PORT_DIR_B 13

#define PORT_PWM_A 3
#define PORT_PWM_B 11

#define PORT_BRAKE_A 9
#define PORT_BRAKE_B 8

#define PORT_CURRENTSENSE_A A0
#define PORT_CURRENTSENSE_B A1

// Bluetoothモジュールと接続用の端子の定義
#define PORT_SOFTUART_RX 10
#define PORT_SOFTUART_TX 7

// Bluetoothモジュールのシリアル通信の速度の定義
#define BLUETOOTH_SERIAL_BAUDRATE  9600

// Software Serialライブラリのインスタンス生成
SoftwareSerial mSerial(PORT_SOFTUART_RX,PORT_SOFTUART_TX);

enum {
  STATE_IDLE,        // コマンド未受信状態
  STATE_RECEIVING    // コマンド受信中状態
};

void procAsciiCommand(unsigned char command);
void procBinaryCommand(byte * command);
void controlMotor(boolean motor_r_fwd,boolean motor_l_fwd,byte motor_r_power,byte motor_l_power);

void setup()
{
  // ポートの初期設定を行う
  pinMode(PORT_DIR_A,OUTPUT);
  pinMode(PORT_DIR_B,OUTPUT);
  pinMode(PORT_PWM_A,OUTPUT);
  pinMode(PORT_PWM_B,OUTPUT);
  pinMode(PORT_BRAKE_A,OUTPUT);
  pinMode(PORT_BRAKE_B,OUTPUT);

  digitalWrite(PORT_DIR_A,LOW);
  digitalWrite(PORT_DIR_B,LOW);
  digitalWrite(PORT_BRAKE_A,LOW);
  digitalWrite(PORT_BRAKE_B,LOW);
  analogWrite(PORT_PWM_A,0);
  analogWrite(PORT_PWM_B,0);

  mSerial.begin(BLUETOOTH_SERIAL_BAUDRATE);
}

void loop()
{
  static unsigned char command = 0;
  static int state = STATE_IDLE;      // ステート管理用変数

  switch(state){
  case STATE_IDLE:
    // コマンド未受信状態
    if(mSerial.available()){
      command = mSerial.read();    // 1バイト目のデータを読み込む
      switch(command){
      case 'A':
      case 'B':
        // コマンドの1バイト目の値と一致した場合、コマンド受信状態に遷移させる
        state = STATE_RECEIVING;  // コマンド受信中状態をセット
        break;
      default:
        break;
      }
    }
    break;
  case STATE_RECEIVING:
    // コマンド受信中状態
    switch(command){  // 受信データの1バイト目(ヘッダ部)の内容による分岐
    case 'A':  // 文字列コマンドの場合
      if(mSerial.available() >= 1){
        state = STATE_IDLE;  // 文字列コマンドのデータを全て受け取れたので、コマンド未受信状態をセット
        procAsciiCommand(mSerial.read());
      }
      break;
    case 'B':  // バイナリコマンドの場合
      if(mSerial.available() >= 4){
        unsigned char receivedData[4];
        state = STATE_IDLE;  // バイナリコマンドのデータを全て受け取れたので、コマンド未受信状態をセット
        int i;
        for(i=0;i<4;i++){
          receivedData[i] = mSerial.read();
          procBinaryCommand(receivedData);
        }
      }
      break;
    default:
      // unknown state
      break;
    }
  }
}

// 文字列コマンドの内容に従いモーター制御を行う
void procAsciiCommand(unsigned char command)
{
  // コマンド文字に応じてモーターを制御する
  switch(command){
  case 'F':
    controlMotor(true,true,255,255);
    break;
  case 'B':
    controlMotor(false,false,255,255);
    break;
  case 'R':
    controlMotor(true,true,255,0);
    break;
  case 'L':
    controlMotor(true,true,0,255);
    break;
  default:
    // unknown command
    break;
  }
  // 1秒間駆動させる
  delay(1000);
  // ブレーキをかける
  controlMotor(true,true,0,0);
}

// バイナリコマンドの内容に従いモーター制御を行う
void procBinaryCommand(byte * command){
  if(command != NULL){
    controlMotor(command[0],command[1],command[2],command[3]);
  }
}

// モーター制御を行う
void controlMotor(boolean motor_r_fwd,boolean motor_l_fwd,byte motor_r_power,byte motor_l_power)
{
  if(motor_r_power == 0){
    digitalWrite(PORT_BRAKE_A,HIGH);
  } 
  else {
    digitalWrite(PORT_BRAKE_A,LOW);
  }    
  if(motor_l_power == 0){
    digitalWrite(PORT_BRAKE_B,HIGH);
  } 
  else {
    digitalWrite(PORT_BRAKE_B,LOW);
  }    

  digitalWrite(PORT_DIR_A,motor_r_fwd == false ? HIGH : LOW);
  digitalWrite(PORT_DIR_B,motor_l_fwd == false ? HIGH : LOW);

  analogWrite(PORT_PWM_A,motor_r_power);
  analogWrite(PORT_PWM_B,motor_l_power);
}

