package jp.maruishi.android.remotecontrollcar03;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.RectF;
import android.util.AttributeSet;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;

public class CarController extends View {
	private static final String TAG = "CarController";
	// マス目のピッチ保存用
	private int gridPitch = 1;
	// 縦横それぞれ11マスとする
	final private int numberOfGrids = 11;

	// 現在操作されているマス目盤上の位置(いずれかまたは両方が-1の時特定のマス目選択無し状態)
	int placeX = -1;
	int placeY = -1;
	
	// 操作時に通知するインターバル
	private int notifyInterval = 100;
	// 前回データ送信した時刻保存用変数
	private long lastNotifyTime = 0;

	ControllerInterface mControllerInterface = null;

	public void setOnControllerChanged(ControllerInterface controllerParams) {
		if (controllerParams != null) {
			mControllerInterface = controllerParams;
		}
	}

	public CarController(Context context, AttributeSet attrs) {
		super(context, attrs);
	}

	@Override
	protected void onSizeChanged(int w, int h, int oldw, int oldh) {
		super.onSizeChanged(w, h, oldw, oldh);

		// 横幅、または高さの小さい方を描画用の処理の横幅と扱う
		int width = w < h ? w : h;

		// 1マスあたりの一辺のドット数を算出する
		gridPitch = width / numberOfGrids;
	}

	@Override
	protected void onDraw(Canvas canvas) {
		drawGrid(canvas);
		if ((placeX != -1) || (placeY != -1)) {
			locateColorTile(canvas, placeX, placeY, true);
		}
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {

		switch (event.getAction()) {
		case MotionEvent.ACTION_MOVE:
			//Log.d(TAG, "ACTION_MOVE " + String.valueOf(event.getX()) + ","
			//		+ String.valueOf(event.getY()));
			setLocation((int) event.getX(), (int) event.getY());
			notifyControllerChanged(false);
			invalidate();
			break;
		case MotionEvent.ACTION_DOWN:
			//Log.d(TAG, "ACTION_DOWN " + String.valueOf(event.getX()) + ","
			//		+ String.valueOf(event.getY()));
			setLocation((int) event.getX(), (int) event.getY());
			notifyControllerChanged(false);
			invalidate();
			break;
		case MotionEvent.ACTION_UP:
			//Log.d(TAG, "ACTION UP");
			unsetLocation();
			notifyControllerChanged(true);
			invalidate();
			break;
		}

		return (true);
	}

	private void setLocation(int x, int y) {
		// ドットをマス目のピッチで割って、タッチされているマス目位置を取得する
		x /= gridPitch;
		y /= gridPitch;

		if ((x >= 0) && (x < numberOfGrids) && (y >= 0) && (y < numberOfGrids)) {
			placeX = x;
			placeY = y;
		}
	}

	private void unsetLocation() {
		placeX = -1;
		placeY = -1;
	}

	// 通知を行う時のインターバルを設定する
	public void setNotifyInterval(int interval){
		notifyInterval = interval;
	}
	
	// 操作内容の変更を通知する
	// 引数 boolean immediate : true  即座に通知する
	//                         false 前回通知時刻からnotifyIntervalミリ秒経過時は通知する
	private void notifyControllerChanged(boolean immediate) {
		if (mControllerInterface != null) {
			// 引数immediateにfalseが設定されて呼び出されたタイミングが、
			// 本関数により前回通知を行ったタイミングから指定時間
			// (notifyIntervalミリ秒)以上経過している場合には、通知を
			// すると判断する。
			if (lastNotifyTime + notifyInterval < System.currentTimeMillis()) {
				immediate = true;
			}
			if(immediate){
				mControllerInterface.onControllerChanged(placeX, placeY);
				lastNotifyTime = System.currentTimeMillis();
			}
		}
	}

	private void drawGrid(Canvas canvas) {
		int c;
		Paint paintBg = new Paint();
		Paint paintGrid = new Paint();

		if (placeX == -1 || placeY == -1) {
			paintBg.setColor(Color.argb(255, 100, 0, 0));
			paintGrid.setColor(Color.argb(255, 180, 0, 180));
		} else {
			paintBg.setColor(Color.argb(255, 50, 100, 200));
			paintGrid.setColor(Color.argb(255, 255, 255, 255));
		}
		RectF rectF = new RectF(0F, 0F, (float) (gridPitch * numberOfGrids),
				(float) (gridPitch * numberOfGrids));
		canvas.drawRect(rectF, paintBg);

		for (c = 0; c <= numberOfGrids; c++) {
			canvas.drawLine(0, c * gridPitch, gridPitch * numberOfGrids, c
					* gridPitch, paintGrid);
			canvas.drawLine(c * gridPitch, 0, c * gridPitch, gridPitch
					* numberOfGrids, paintGrid);
		}
	}

	private void locateColorTile(Canvas canvas, int column, int line,
			boolean set) {
		Paint paint = new Paint();
		if (set == true) {
			paint.setColor(Color.argb(255, 255, 255, 0));
		} else {
			paint.setColor(Color.argb(255, 0, 0, 0));
		}

		paint.setStyle(Paint.Style.FILL);

		canvas.drawRect(column * gridPitch, 0, (column + 1) * gridPitch,
				numberOfGrids * gridPitch, paint);
		canvas.drawRect(0, line * gridPitch, numberOfGrids * gridPitch,
				(line + 1) * gridPitch, paint);
		// canvas.drawRect(column * gridPitch, line * gridPitch, (column + 1)
		// * gridPitch - 1, (line + 1) * gridPitch - 1, paint);
	}
}
